// FileChangeNotify.cpp: implementation of the CFileChangeNotify class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <string>
#include <iterator>

#include "FileChangeNotify.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileChangeNotify::CFileChangeNotify()
{
	m_isWaiting = false;
	m_notifyCallback = NULL;
	m_thread = NULL;
}	

CFileChangeNotify::~CFileChangeNotify()
{
	stopNotify();
}

int CFileChangeNotify::addWatch(LPCSTR lpPathName, BOOL bWatchSubtree, DWORD dwNotifyFilter)
{
	if(m_isWaiting == true) 
		return -1;
	HANDLE hand = FindFirstChangeNotification(lpPathName, bWatchSubtree, dwNotifyFilter);
	if(hand == INVALID_HANDLE_VALUE)
		return -1;
	m_chHandle.push_back(hand);
	return 0;
}

DWORD WINAPI watching_thread(LPVOID param)
{
	CFileChangeNotify *pNotify = (CFileChangeNotify *)param;
	
	list<HANDLE> &_hand = pNotify->m_chHandle;
	list<HANDLE>::iterator iter;
	list<HANDLE>::difference_type diff;

	diff = distance(_hand.begin(), _hand.end());

	HANDLE *pHH = new HANDLE[diff];

	int loc = 0;
	for(iter = _hand.begin(); iter != _hand.end(); iter++) {
		pHH[loc++] = *iter;
	}

	DWORD status;
	while(true) { 
		// Wait for notification.
		status = WaitForMultipleObjects(diff, pHH, FALSE, INFINITE); 
		int idx = status - WAIT_OBJECT_0;
		pNotify->m_notifyCallback(idx);
		if(FindNextChangeNotification(pHH[idx]) == FALSE) {
			break;
		}
	}

	delete [] pHH;

	return 0;
}

int CFileChangeNotify::startNotify()
{
	if(m_isWaiting == true) 
		return -1;
	if(m_notifyCallback == NULL)
		return -1;

	m_isWaiting = true;		// enter the watching mode

	m_thread = CreateThread(NULL, NULL, watching_thread, (LPVOID)this, NULL, NULL);
	if(m_thread == INVALID_HANDLE_VALUE)
		return -1;

	return 0;
}

void CFileChangeNotify::stopNotify()
{
	if(m_isWaiting == false)
		return;
	m_isWaiting = false;

	if(m_thread != NULL) {
		TerminateThread(m_thread, 0);
		m_thread = NULL;
	}

	m_notifyCallback = NULL;
	m_chHandle.erase(m_chHandle.begin(), m_chHandle.end());
}

